"""Data models for trading operations."""

from dataclasses import dataclass, field, fields
from decimal import Decimal
from enum import Enum
from typing import TypedDict


@dataclass
class AccountOverview:
    """账户资金总览数据类"""

    Djzj: float  # 冻结资金
    Dryk: float  # 当日盈亏
    Kqzj: float  # 可取资金
    Kyzj: float  # 可用资金
    Ljyk: float  # 累计盈亏
    Money_type: str  # 货币类型
    RMBZzc: float  # 人民币总资产
    Zjye: float  # 资金余额
    Zxsz: float  # 证券市值
    Zzc: float  # 总资产

    def __post_init__(self) -> None:
        # 确保所有数值字段都是浮点数
        for v in fields(self):
            if v.name not in ["Money_type"]:
                value = getattr(self, v.name)
                if isinstance(value, str):
                    setattr(self, v.name, float(value))

    def __str__(self) -> str:
        """格式化打印账户信息"""
        title = "账户资金总览"
        separator = "=" * 50

        # 创建格式化字符串
        lines = [
            separator,
            f"{title:^50}",
            separator,
            f"{'货币类型:':<20} {self.Money_type:>30}",
            f"{'总资产： ':<20} {self.Zzc:>30,.2f}",
            f"{'证券市值:':<20} {self.Zxsz:>30,.2f}",
            f"{'可用资金:':<20} {self.Kyzj:>30,.2f}",
            f"{'可取资金:':<20} {self.Kqzj:>30,.2f}",
            f"{'冻结资金:':<20} {self.Djzj:>30,.2f}",
            f"{'累计盈亏:':<20} {self.Ljyk:>30,.2f}",
            f"{'当日盈亏:':<20} {self.Dryk:>30,.2f}",
            f"{'资金余额:':<20} {self.Zjye:>30,.2f}",
            separator,
        ]

        return "\n".join(lines)


@dataclass
class Position:
    """持仓明细数据类"""

    Bz: str  # 币种
    Cbjg: Decimal  # 成本价格
    Cbjgex: Decimal  # 成本价格(精确)
    Ckcb: Decimal  # 持仓成本
    Ckcbj: Decimal  # 持仓成本价
    Ckyk: Decimal  # 持仓盈亏
    Cwbl: Decimal  # 仓位比例
    Djsl: int  # 冻结数量
    Dqcb: Decimal  # 当前成本
    Dryk: Decimal  # 当日盈亏
    Drykbl: Decimal  # 当日盈亏比例
    Gddm: str  # 股东代码
    Gfmcdj: int  # 股份卖出冻结
    Gfmrjd: int  # 股份买入解冻
    Gfssmmce: int  # 股份上市末码额
    Gfye: int  # 股份余额
    Jgbm: str  # 机构编码
    Khdm: str  # 客户代码
    Ksssl: int  # 可售数量
    Kysl: int  # 可用数量
    Ljyk: Decimal  # 累计盈亏
    Market: str  # 市场
    Mrssc: int  # 买入市场
    Sssl: int  # 申购数量
    Szjsbs: int  # 市值计算标识
    Ykbl: Decimal  # 盈亏比例
    Zjzh: str  # 资金账号
    Zqdm: str  # 证券代码
    Zqlx: str  # 证券类型
    Zqlxmc: str  # 证券类型名称
    Zqmc: str  # 证券名称
    Zqsl: int  # 证券数量
    Ztmc: int  # 状态码
    Ztmr: int  # 状态码
    Zxjg: Decimal  # 最新价格
    Zxsz: Decimal  # 最新市值
    zqzwqc: str  # 证券中文全称
    zxsznew: Decimal  # 最新市值(新)

    def __post_init__(self) -> None:
        # 转换数值类型
        decimal_fields = [
            "Cbjg",
            "Cbjgex",
            "Ckcb",
            "Ckcbj",
            "Ckyk",
            "Cwbl",
            "Dqcb",
            "Dryk",
            "Drykbl",
            "Ljyk",
            "Ykbl",
            "Zxjg",
            "Zxsz",
            "zxsznew",
        ]

        int_fields = [
            "Djsl",
            "Gfmcdj",
            "Gfmrjd",
            "Gfssmmce",
            "Gfye",
            "Ksssl",
            "Kysl",
            "Mrssc",
            "Sssl",
            "Szjsbs",
            "Zqsl",
            "Ztmc",
            "Ztmr",
        ]

        for field_name in decimal_fields:
            value = getattr(self, field_name)
            if isinstance(value, str):
                setattr(self, field_name, Decimal(value))

        for field_name in int_fields:
            value = getattr(self, field_name)
            if isinstance(value, str):
                setattr(self, field_name, int(value))


@dataclass
class Portfolio:
    """持仓组合数据类，包含多个持仓"""

    positions: list[Position] = field(default_factory=list)

    def add_position(self, position: Position) -> None:
        """添加持仓"""
        self.positions.append(position)

    def __str__(self) -> str:
        """格式化打印持仓信息"""
        if not self.positions:
            return "暂无持仓"

        # 表头 - 使用固定宽度确保对齐
        headers = [
            "证券名称",
            "证券代码",
            "当前价",
            "持仓数量",
            "成本价",
            "当前市值",
            "累计盈亏",
            "盈亏比例",
        ]
        col_widths = [12, 10, 10, 12, 10, 12, 12, 12]  # 每列宽度

        # 构建表头
        header_line = "".join(
            f"{header:<{width}}"
            for header, width in zip(headers, col_widths, strict=False)
        )
        separator = "-" * sum(col_widths)

        # 构建表格行
        rows = [header_line, separator]
        total_market_value = Decimal("0")
        total_profit_loss = Decimal("0")

        for pos in self.positions:
            # 格式化数据
            name = pos.Zqmc if len(pos.Zqmc) <= 10 else pos.Zqmc[:7] + "..."
            code = pos.Zqdm
            price = f"{pos.Zxjg:.3f}"
            quantity = f"{pos.Zqsl:,}"
            cost = f"{pos.Cbjg:.3f}"
            market_value = f"{pos.Zxsz:,.2f}"
            profit_loss = f"{pos.Ljyk:,.2f}"
            profit_ratio = f"{pos.Ykbl:.2%}"

            # 构建行
            row_data = [
                name,
                code,
                price,
                quantity,
                cost,
                market_value,
                profit_loss,
                profit_ratio,
            ]
            row = "".join(
                f"{data:<{width}}"
                for data, width in zip(row_data, col_widths, strict=False)
            )
            rows.append(row)

            # 累加总市值和总盈亏
            total_market_value += pos.Zxsz
            total_profit_loss += pos.Ljyk

        # 添加总计行
        rows.append(separator)
        total_row_data = [
            "总计",
            "",
            "",
            "",
            "",
            f"{total_market_value:,.2f}",
            f"{total_profit_loss:,.2f}",
            "",
        ]
        total_row = "".join(
            f"{data:<{width}}"
            for data, width in zip(total_row_data, col_widths, strict=False)
        )
        rows.append(total_row)

        return "\n".join(rows)


class OrderType(Enum):
    """Order types for trading"""

    BUY = "buy"
    SELL = "sell"


class OrderStatus(Enum):
    """Order status types"""

    PENDING = "pending"
    FILLED = "filled"
    CANCELLED = "cancelled"
    REJECTED = "rejected"


@dataclass
class Order:
    """Represents a trading order"""

    symbol: str
    order_type: OrderType
    quantity: float
    price: float
    order_id: str | None = None
    status: OrderStatus = OrderStatus.PENDING


class AccountInfo(TypedDict):
    """Represents account information"""

    username: str
    account_overview: AccountOverview
    portfolio: Portfolio


class MarketData(TypedDict):
    """Represents market data for a symbol"""

    symbol: str
    last_price: float
    bid_price: float
    ask_price: float
    volume: int
    timestamp: str


# 使用示例
if __name__ == "__main__":
    # 从JSON数据创建对象
    json_data: dict[str, object] = {
        "Status": 0,
        "Count": 0,
        "Data": [
            {
                "Djzj": "0.00",
                "Dryk": "0.00",
                "Kqzj": "447.26",
                "Kyzj": "452.23",
                "Ljyk": "-1540.00",
                "Money_type": "RMB",
                "RMBZzc": "152788.23",
                "Zjye": "447.26",
                "Zxsz": "152340.97",
                "Zzc": "152788.23",
            }
        ],
        "Errcode": 0,
    }

    # 提取Data中的第一个元素
    data = json_data["Data"][0]  # type: ignore

    # 创建AccountOverview实例
    account = AccountOverview(
        Djzj=data["Djzj"],
        Dryk=data["Dryk"],
        Kqzj=data["Kqzj"],
        Kyzj=data["Kyzj"],
        Ljyk=data["Ljyk"],
        Money_type=data["Money_type"],
        RMBZzc=data["RMBZzc"],
        Zjye=data["Zjye"],
        Zxsz=data["Zxsz"],
        Zzc=data["Zzc"],
    )

    # 打印格式化后的账户信息
    print(account)

    # 从JSON数据创建持仓对象
    json_data_positions: dict[str, object] = {
        "Status": 0,
        "Count": 0,
        "Data": [
            {
                "positions": [
                    {
                        "Bz": "RMB",
                        "Cbjg": "1.000",
                        "Cbjgex": "1.000",
                        "Ckcb": "2000.00",
                        "Ckcbj": "1.000",
                        "Ckyk": "-1540.00",
                        "Cwbl": "0.00301",
                        "Djsl": "0",
                        "Dqcb": "2000.00",
                        "Dryk": "0.00",
                        "Drykbl": "0.000000",
                        "Gddm": "0235371788",
                        "Gfmcdj": "0",
                        "Gfmrjd": "0",
                        "Gfssmmce": "0",
                        "Gfye": "2000",
                        "Jgbm": "5408",
                        "Khdm": "540860219538",
                        "Ksssl": "2000",
                        "Kysl": "2000",
                        "Ljyk": "-1540.00",
                        "Market": "TA",
                        "Mrssc": "0",
                        "Sssl": "0",
                        "Szjsbs": "1",
                        "Ykbl": "-0.770000",
                        "Zjzh": "540868886633",
                        "Zqdm": "400251",
                        "Zqlx": "0",
                        "Zqlxmc": "0",
                        "Zqmc": "海印5",
                        "Zqsl": "2000",
                        "Ztmc": "0",
                        "Ztmr": "0",
                        "Zxjg": "0.230",
                        "Zxsz": "460.00",
                        "zqzwqc": "海印5",
                        "zxsznew": "460.00",
                    },
                    {
                        "Bz": "RMB",
                        "Cbjg": "8.830",
                        "Cbjgex": "8.830",
                        "Ckcb": "151876.00",
                        "Ckcbj": "8.830",
                        "Ckyk": "0.00",
                        "Cwbl": "0.99403",
                        "Djsl": "0",
                        "Dqcb": "151876.00",
                        "Dryk": "0.00",
                        "Drykbl": "0.000000",
                        "Gddm": "0235371788",
                        "Gfmcdj": "0",
                        "Gfmrjd": "0",
                        "Gfssmmce": "0",
                        "Gfye": "17200",
                        "Jgbm": "5408",
                        "Khdm": "540860219538",
                        "Ksssl": "17200",
                        "Kysl": "17200",
                        "Ljyk": "0.00",
                        "Market": "B",
                        "Mrssc": "0",
                        "Sssl": "0",
                        "Szjsbs": "1",
                        "Ykbl": "0.000000",
                        "Zjzh": "540868886633",
                        "Zqdm": "920100",
                        "Zqlx": "J",
                        "Zqlxmc": "J",
                        "Zqmc": "三协电机",
                        "Zqsl": "17200",
                        "Ztmc": "0",
                        "Ztmr": "0",
                        "Zxjg": "8.830",
                        "Zxsz": "151876.00",
                        "zqzwqc": "三协电机",
                        "zxsznew": "151876.00",
                    },
                ]
            }
        ],
        "Errcode": 0,
    }

    # 创建持仓组合
    portfolio = Portfolio()

    # 提取并添加所有持仓
    for pos_data in json_data_positions["Data"][0]["positions"]:  # type: ignore
        position = Position(**pos_data)
        portfolio.add_position(position)

    print(portfolio)



"""Tests for the Eastmoney Trading Agent"""

import os
from unittest.mock import MagicMock, patch

from loguru import logger

from emta.core.agent import TradingAgent
from emta.models.trading import OrderStatus, OrderType


class TestTradingAgent:
    """Tests for the TradingAgent class"""

    def test_init(self) -> None:
        """Test TradingAgent initialization"""
        agent = TradingAgent()
        assert agent.username is None
        assert agent.password is None
        assert agent.is_logged_in is False

    def test_init_with_credentials(self) -> None:
        """Test TradingAgent initialization with credentials"""
        agent = TradingAgent("test_user", "test_pass")
        assert agent.username == "test_user"
        assert agent.password == "test_pass"

    @patch("emta.auth.client.AuthClient.login")
    @patch("emta.auth.client.AuthClient._get_captcha")
    @patch("emta.api.client.APIClient.get_asset_and_position")
    def test_login(
        self,
        mock_asset_position: MagicMock,
        mock_captcha: MagicMock,
        mock_auth_login: MagicMock,
    ) -> None:
        """Test TradingAgent login with credentials from environment variables"""
        logger.info("test_login")

        # Mock the authentication client
        mock_auth_login.return_value = (True, {"Status": 0, "Message": "Success"})

        # Mock asset and position data with proper structure
        mock_asset_position.return_value = {
            "Data": [
                {
                    "Djzj": 100000,
                    "Dryk": 5000,
                    "Kqzj": 95000,
                    "Kyzj": 80000,
                    "Ljyk": 10000,
                    "Money_type": "RMB",
                    "RMBZzc": 120000,
                    "Zjye": 100000,
                    "Zxsz": 110000,
                    "Zzc": 120000,
                    "positions": [],
                }
            ]
        }

        # Mock the captcha
        mock_captcha.return_value = None

        agent = TradingAgent(
            os.getenv("EM_USERNAME", "test_user"),
            os.getenv("EM_PASSWORD", "test_pass"),
        )
        result = agent.login()

        assert result is True
        assert agent.is_logged_in is True
        # Verify that login was called with the correct parameters
        mock_auth_login.assert_called_once_with(
            os.getenv("EM_USERNAME", "test_user"),
            os.getenv("EM_PASSWORD", "test_pass"),
            30,
            3,
            1.0,
        )

    @patch("emta.auth.client.AuthClient.login")
    @patch("emta.auth.client.AuthClient._get_captcha")
    @patch("emta.api.client.APIClient.get_asset_and_position")
    def test_login_success(
        self,
        mock_asset_position: MagicMock,
        mock_captcha: MagicMock,
        mock_auth_login: MagicMock,
    ) -> None:
        """Test successful login"""
        # Mock the authentication client
        mock_auth_login.return_value = (True, {"Status": 0, "Message": "Success"})

        # Mock asset and position data with proper structure
        mock_asset_position.return_value = {
            "Data": [
                {
                    "Djzj": 100000,
                    "Dryk": 5000,
                    "Kqzj": 95000,
                    "Kyzj": 80000,
                    "Ljyk": 10000,
                    "Money_type": "RMB",
                    "RMBZzc": 120000,
                    "Zjye": 100000,
                    "Zxsz": 110000,
                    "Zzc": 120000,
                    "positions": [],
                }
            ]
        }

        # Mock the captcha
        mock_captcha.return_value = None

        agent = TradingAgent("test_user", "test_pass")
        result = agent.login()

        assert result is True
        assert agent.is_logged_in is True
        # Verify that login was called with the correct parameters
        mock_auth_login.assert_called_once_with("test_user", "test_pass", 30, 3, 1.0)

    def test_login_failure_missing_credentials(self) -> None:
        """Test login failure when credentials are missing"""
        agent = TradingAgent()
        result = agent.login()
        assert result is False
        assert agent.is_logged_in is False

    @patch("emta.auth.client.AuthClient.login")
    @patch("emta.auth.client.AuthClient._get_captcha")
    def test_login_failure_api_error(
        self, mock_captcha: MagicMock, mock_auth_login: MagicMock
    ) -> None:
        """Test login failure due to API error"""
        # Mock the authentication client to return failure
        mock_auth_login.return_value = (
            False,
            {"Status": -1, "Message": "Invalid credentials"},
        )

        # Mock the captcha
        mock_captcha.return_value = None

        agent = TradingAgent("test_user", "test_pass")
        result = agent.login()

        assert result is False
        assert agent.is_logged_in is False
        # Verify that login was called with the correct parameters
        mock_auth_login.assert_called_once_with("test_user", "test_pass", 30, 3, 1.0)

    @patch("emta.auth.client.AuthClient.login")
    @patch("emta.auth.client.AuthClient._get_captcha")
    @patch("emta.auth.client.AuthClient.logout")
    def test_logout(
        self,
        mock_auth_logout: MagicMock,
        mock_captcha: MagicMock,
        mock_auth_login: MagicMock,
    ) -> None:
        """Test logout functionality"""
        # Mock the authentication client
        mock_auth_login.return_value = (True, {"Status": 0, "Message": "Success"})
        mock_auth_logout.return_value = None

        # Mock asset and position data with proper structure
        with patch(
            "emta.api.client.APIClient.get_asset_and_position"
        ) as mock_asset_position:
            mock_asset_position.return_value = {
                "Data": [
                    {
                        "Djzj": 100000,
                        "Dryk": 5000,
                        "Kqzj": 95000,
                        "Kyzj": 80000,
                        "Ljyk": 10000,
                        "Money_type": "RMB",
                        "RMBZzc": 120000,
                        "Zjye": 100000,
                        "Zxsz": 110000,
                        "Zzc": 120000,
                        "positions": [],
                    }
                ]
            }

            # Mock the captcha
            mock_captcha.return_value = None

            agent = TradingAgent("test_user", "test_pass")
            agent.login()
            # Set the validate_key to simulate successful login
            agent.auth_client.validate_key = "test_validate_key"
            assert agent.is_logged_in is True
            # Verify that login was called with the correct parameters
            mock_auth_login.assert_called_once_with(
                "test_user", "test_pass", 30, 3, 1.0
            )

            agent.logout()
            assert agent.is_logged_in is False
            assert agent.account_info == []
            mock_auth_logout.assert_called_once()

    @patch("emta.auth.client.AuthClient.login")
    @patch("emta.auth.client.AuthClient._get_captcha")
    def test_get_account_info(
        self, mock_captcha: MagicMock, mock_auth_login: MagicMock
    ) -> None:
        """Test getting account information"""
        # Mock the authentication client
        mock_auth_login.return_value = (True, {"Status": 0, "Message": "Success"})

        # Mock the captcha
        mock_captcha.return_value = None

        # Mock asset and position data with proper structure
        with patch(
            "emta.api.client.APIClient.get_asset_and_position"
        ) as mock_asset_position:
            mock_asset_position.return_value = {
                "Data": [
                    {
                        "Djzj": 100000,
                        "Dryk": 5000,
                        "Kqzj": 95000,
                        "Kyzj": 80000,
                        "Ljyk": 10000,
                        "Money_type": "RMB",
                        "RMBZzc": 120000,
                        "Zjye": 100000,
                        "Zxsz": 110000,
                        "Zzc": 120000,
                        "positions": [],
                    }
                ]
            }

            agent = TradingAgent("test_user", "test_pass")
            agent.login()
            # Set the validate_key to simulate successful login
            agent.auth_client.validate_key = "test_validate_key"

            account_info = agent.get_account_info()
            assert isinstance(account_info, dict)
            assert "username" in account_info
            assert "account_balance" in account_info
            # Verify that login was called with the correct parameters
            mock_auth_login.assert_called_once_with(
                "test_user", "test_pass", 30, 3, 1.0
            )

    def test_get_account_info_not_logged_in(self) -> None:
        """Test getting account info when not logged in"""
        agent = TradingAgent()
        account_info = agent.get_account_info()
        assert account_info == []

    @patch("emta.auth.client.AuthClient.login")
    @patch("emta.auth.client.AuthClient._get_captcha")
    def test_place_order(
        self, mock_captcha: MagicMock, mock_auth_login: MagicMock
    ) -> None:
        """Test placing an order"""
        # Mock the authentication client
        mock_auth_login.return_value = (True, {"Status": 0, "Message": "Success"})

        # Mock the captcha
        mock_captcha.return_value = None

        # Mock asset and position data with proper structure
        with patch(
            "emta.api.client.APIClient.get_asset_and_position"
        ) as mock_asset_position:
            mock_asset_position.return_value = {
                "Data": [
                    {
                        "Djzj": 100000,
                        "Dryk": 5000,
                        "Kqzj": 95000,
                        "Kyzj": 80000,
                        "Ljyk": 10000,
                        "Money_type": "RMB",
                        "RMBZzc": 120000,
                        "Zjye": 100000,
                        "Zxsz": 110000,
                        "Zzc": 120000,
                        "positions": [],
                    }
                ]
            }

            agent = TradingAgent("test_user", "test_pass")
            agent.login()

            # Set the validate_key to simulate successful login
            agent.auth_client.validate_key = "test_validate_key"

            order_id = agent.place_order("SH600000", OrderType.BUY, 100, 12.5)
            assert order_id is not None
            assert isinstance(order_id, str)
            # Verify that login was called with the correct parameters
            mock_auth_login.assert_called_once_with(
                "test_user", "test_pass", 30, 3, 1.0
            )

    def test_place_order_not_logged_in(self) -> None:
        """Test placing an order when not logged in"""
        agent = TradingAgent()
        order_id = agent.place_order("SH600000", OrderType.BUY, 100, 12.5)
        assert order_id is None

    @patch("emta.auth.client.AuthClient.login")
    @patch("emta.auth.client.AuthClient._get_captcha")
    def test_cancel_order(
        self, mock_captcha: MagicMock, mock_auth_login: MagicMock
    ) -> None:
        """Test cancelling an order"""
        # Mock the authentication client
        mock_auth_login.return_value = (True, {"Status": 0, "Message": "Success"})

        # Mock the captcha
        mock_captcha.return_value = None

        # Mock asset and position data with proper structure
        with patch(
            "emta.api.client.APIClient.get_asset_and_position"
        ) as mock_asset_position:
            mock_asset_position.return_value = {
                "Data": [
                    {
                        "Djzj": 100000,
                        "Dryk": 5000,
                        "Kqzj": 95000,
                        "Kyzj": 80000,
                        "Ljyk": 10000,
                        "Money_type": "RMB",
                        "RMBZzc": 120000,
                        "Zjye": 100000,
                        "Zxsz": 110000,
                        "Zzc": 120000,
                        "positions": [],
                    }
                ]
            }

            agent = TradingAgent("test_user", "test_pass")
            agent.login()
            # Set the validate_key to simulate successful login
            agent.auth_client.validate_key = "test_validate_key"

            result = agent.cancel_order("test_order_id")
            assert result is True
            # Verify that login was called with the correct parameters
            mock_auth_login.assert_called_once_with(
                "test_user", "test_pass", 30, 3, 1.0
            )

    def test_cancel_order_not_logged_in(self) -> None:
        """Test cancelling an order when not logged in"""
        agent = TradingAgent()
        result = agent.cancel_order("test_order_id")
        assert result is False

    @patch("emta.auth.client.AuthClient.login")
    @patch("emta.auth.client.AuthClient._get_captcha")
    def test_get_order_status(
        self, mock_captcha: MagicMock, mock_auth_login: MagicMock
    ) -> None:
        """Test getting order status"""
        # Mock the authentication client
        mock_auth_login.return_value = (True, {"Status": 0, "Message": "Success"})

        # Mock the captcha
        mock_captcha.return_value = None

        # Mock asset and position data with proper structure
        with patch(
            "emta.api.client.APIClient.get_asset_and_position"
        ) as mock_asset_position:
            mock_asset_position.return_value = {
                "Data": [
                    {
                        "Djzj": 100000,
                        "Dryk": 5000,
                        "Kqzj": 95000,
                        "Kyzj": 80000,
                        "Ljyk": 10000,
                        "Money_type": "RMB",
                        "RMBZzc": 120000,
                        "Zjye": 100000,
                        "Zxsz": 110000,
                        "Zzc": 120000,
                        "positions": [],
                    }
                ]
            }

            agent = TradingAgent("test_user", "test_pass")
            agent.login()
            # Set the validate_key to simulate successful login
            agent.auth_client.validate_key = "test_validate_key"

            status = agent.get_order_status("test_order_id")
            assert status is not None
            assert isinstance(status, OrderStatus)
            # Verify that login was called with the correct parameters
            mock_auth_login.assert_called_once_with(
                "test_user", "test_pass", 30, 3, 1.0
            )

    def test_get_order_status_not_logged_in(self) -> None:
        """Test getting order status when not logged in"""
        agent = TradingAgent()
        status = agent.get_order_status("test_order_id")
        assert status is None

    @patch("emta.auth.client.AuthClient.login")
    @patch("emta.auth.client.AuthClient._get_captcha")
    def test_get_market_data(
        self, mock_captcha: MagicMock, mock_auth_login: MagicMock
    ) -> None:
        """Test getting market data"""
        # Mock the authentication client
        mock_auth_login.return_value = (True, {"Status": 0, "Message": "Success"})

        # Mock the captcha
        mock_captcha.return_value = None

        # Mock asset and position data with proper structure
        with patch(
            "emta.api.client.APIClient.get_asset_and_position"
        ) as mock_asset_position:
            mock_asset_position.return_value = {
                "Data": [
                    {
                        "Djzj": 100000,
                        "Dryk": 5000,
                        "Kqzj": 95000,
                        "Kyzj": 80000,
                        "Ljyk": 10000,
                        "Money_type": "RMB",
                        "RMBZzc": 120000,
                        "Zjye": 100000,
                        "Zxsz": 110000,
                        "Zzc": 120000,
                        "positions": [],
                    }
                ]
            }

            agent = TradingAgent("test_user", "test_pass")
            agent.login()
            # Set the validate_key to simulate successful login
            agent.auth_client.validate_key = "test_validate_key"

            market_data = agent.get_market_data("SH600000")
            assert isinstance(market_data, dict)
            assert "symbol" in market_data
            assert "last_price" in market_data
            # Verify that login was called with the correct parameters
            mock_auth_login.assert_called_once_with(
                "test_user", "test_pass", 30, 3, 1.0
            )

    def test_get_market_data_not_logged_in(self) -> None:
        """Test getting market data when not logged in"""
        agent = TradingAgent()
        market_data = agent.get_market_data("SH600000")
        assert market_data == {}



"""API client for Eastmoney trading operations."""

from typing import Any

import httpx
from loguru import logger

from emta.models.trading import OrderType

from ..models.exceptions import TradingError

# Base headers for API requests
BASE_HEADERS: dict[str, str] = {
    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/114.0.0.0 Safari/537.36",
    "Origin": "https://jywg.18.cn",
    "Host": "jywg.18.cn",
}


class APIClient:
    """Handles API requests to Eastmoney trading services"""

    def __init__(self, session: httpx.Client):
        """Initialize the API client.

        Args:
            session: HTTP client session
        """
        self.session = session
        self.logger = logger

    def _check_response(self, resp: httpx.Response) -> None:
        """Check if response is successful.

        Args:
            resp: HTTP response to check

        Raises:
            TradingError: If response status is not 200
        """
        if resp.status_code != 200:
            self.logger.error(
                f"Request {resp.url} failed, code={resp.status_code}, response={resp.text}"
            )
            raise TradingError(f"API request failed with status {resp.status_code}")

    def query_something(
        self, url: str, validate_key: str, req_data: dict[str, Any] | None = None
    ) -> httpx.Response:
        """通用查询函数

        :param url: 请求url
        :param validate_key: 验证key
        :param req_data: 请求提交数据,可选
        :return: HTTP响应
        """
        full_url = f"{url}{validate_key}"
        if req_data is None:
            req_data = {
                "qqhs": 100,
                "dwc": "",
            }
        headers = BASE_HEADERS.copy()
        headers["X-Requested-With"] = "XMLHttpRequest"
        self.logger.debug(f"(url={full_url}), (data={req_data})")
        resp = self.session.post(full_url, headers=headers, data=req_data)
        self._check_response(resp)
        return resp

    def get_asset_and_position(self, validate_key: str) -> dict[str, Any]:
        """Get asset and position information.

        Args:
            validate_key: Validation key for the session

        Returns:
            Dictionary containing asset and position data
        """
        url = "https://jywg.18.cn/Com/queryAssetAndPositionV1?validatekey="
        resp = self.query_something(url, validate_key)
        return resp.json()  # type: ignore[no-any-return]

    def create_order(
        self,
        validate_key: str,
        stock_code: str,
        trade_type: OrderType,
        market: str,
        price: float,
        amount: int,
    ) -> dict[str, Any]:
        """交易接口, 买入或卖出.

        :param str stock_code: 股票代码
        :param str trade_type: 交易方向,B for buy, S for sell
        :param str market: 股票市场,HA 上海, SA
        :param float price: 股票价格
        :param int amount: 买入/卖出数量
        """
        req_data = {
            "stockCode": stock_code,
            "tradeType": trade_type,
            "zqmc": "",
            "market": market,
            "price": price,
            "amount": amount,
        }
        url = "https://jywg.18.cn/Trade/SubmitTradeV2?validatekey="
        resp = self.query_something(url, validate_key, req_data=req_data)
        if resp:
            self.logger.info(resp.json())
        return resp.json()  # type: ignore[no-any-return]



"""Main trading agent implementation."""

from typing import Any

import httpx
from loguru import logger

from ..api.client import APIClient
from ..auth.client import AuthClient
from ..models.trading import (
    AccountInfo,
    AccountOverview,
    MarketData,
    OrderStatus,
    OrderType,
    Portfolio,
    Position,
)


class TradingAgent:
    """Eastmoney Trading Agent

    This class provides methods for interacting with Eastmoney trading services,
    including login, placing orders, checking account status, and retrieving market data.
    """

    def __init__(self, username: str | None = None, password: str | None = None):
        """Initialize the trading agent

        Args:
            username: Eastmoney account username
            password: Eastmoney account password
        """
        self.username = username
        self.password = password
        self.is_logged_in = False
        self.account_info: list[AccountInfo] = []
        self.logger = logger
        self.session = httpx.Client()
        self.auth_client = AuthClient(self.session)
        self.api_client = APIClient(self.session)

    def login(
        self,
        username: str | None = None,
        password: str | None = None,
        duration: int = 30,
        max_retries: int = 3,
        retry_delay: float = 1.0,
    ) -> bool:
        """Login to Eastmoney account

        Args:
            username: Eastmoney account username (optional if provided in constructor)
            password: Eastmoney account password (optional if provided in constructor)
            duration: Eastmoney account login session duration in minutes (default: 30)
            max_retries: Maximum number of retry attempts (default: 3)
            retry_delay: Delay between retry attempts in seconds (default: 1.0)

        Returns:
            bool: True if login successful, False otherwise

        Raises:
            LoginError: If login fails due to network or authentication issues
        """
        # Use provided credentials or fallback to instance variables
        login_username = username or self.username
        login_password = password or self.password

        if not login_username or not login_password:
            self.logger.error("Username and password are required for login")
            return False

        self.username = login_username
        self.password = login_password

        try:
            success, response = self.auth_client.login(
                login_username, login_password, duration, max_retries, retry_delay
            )
            if success:
                self.is_logged_in = True
                # Get asset and position data
                asset_pos = self.api_client.get_asset_and_position(
                    self.auth_client.validate_key or ""
                )
                self.logger.info(asset_pos)

                # Handle the case where asset_pos might not have "Data" key
                if "Data" in asset_pos:
                    for data in asset_pos["Data"]:
                        # Create AccountOverview instance
                        account = AccountOverview(
                            Djzj=data.get("Djzj", 0),
                            Dryk=data.get("Dryk", 0),
                            Kqzj=data.get("Kqzj", 0),
                            Kyzj=data.get("Kyzj", 0),
                            Ljyk=data.get("Ljyk", 0),
                            Money_type=data.get("Money_type", ""),
                            RMBZzc=data.get("RMBZzc", 0),
                            Zjye=data.get("Zjye", 0),
                            Zxsz=data.get("Zxsz", 0),
                            Zzc=data.get("Zzc", 0),
                        )
                        print(account)

                        portfolio = Portfolio()

                        # Extract and add all positions
                        positions = data.get("positions", [])
                        for pos_data in positions:
                            position = Position(**pos_data)
                            portfolio.add_position(position)
                            print(portfolio)

                        account_info: AccountInfo = {
                            "username": login_username,
                            "account_overview": account,
                            "portfolio": portfolio,
                        }
                        self.account_info.append(account_info)
            else:
                self.logger.info(response)
            return success
        except Exception as e:
            self.logger.error(f"Login failed: {e}")
            self.is_logged_in = False
            return False

    def logout(self) -> None:
        """Logout from Eastmoney account"""
        self.is_logged_in = False
        self.account_info = []
        self.auth_client.logout()
        self.logger.info("Logged out successfully")

    def get_account_info(self) -> dict[str, Any] | list[AccountInfo]:
        """Get account information

        Returns:
            Dict containing account information
        """
        if not self.is_logged_in:
            self.logger.warning("User not logged in")
            return []

        # For backward compatibility, return a dict for the first account if available
        if self.account_info:
            first_account = self.account_info[0]
            # Since AccountInfo is a TypedDict, first_account is already a dict
            return {
                "username": first_account["username"],
                "account_balance": first_account["account_overview"].Zjye
                if first_account["account_overview"]
                else 0,
                "portfolio": first_account["portfolio"],
            }

        return []

    def place_order(
        self, stock_code: str, trade_type: OrderType, amount: int, price: float
    ) -> str | None:
        """Place a trading order

        Args:
            stock_code: Stock symbol
            trade_type: Order type (BUY or SELL)
            amount: Number of shares
            price: Order price

        Returns:
            Order ID if successful, None otherwise
        """
        if not self.is_logged_in or not self.auth_client.validate_key:
            self.logger.error("User not logged in")
            return None

        self.logger.info(
            f"Placing {trade_type.value} order for {stock_code}: "
            f"{amount} shares at {price}"
        )
        market = ""
        resp = self.api_client.create_order(
            self.auth_client.validate_key, stock_code, trade_type, market, price, amount
        )
        self.logger.info(resp)
        order_id = f"order_{stock_code}_{trade_type.value}_{amount}_{price}"

        self.logger.info(f"Order placed successfully with ID: {order_id}")
        return order_id

    def cancel_order(self, order_id: str) -> bool:
        """Cancel an existing order

        Args:
            order_id: ID of the order to cancel

        Returns:
            True if cancellation successful, False otherwise
        """
        if not self.is_logged_in:
            self.logger.error("User not logged in")
            return False

        self.logger.info(f"Cancelling order: {order_id}")

        # TODO: Implement actual order cancellation with Eastmoney API
        # This is a placeholder implementation
        self.logger.info(f"Order {order_id} cancelled successfully")
        return True

    def get_order_status(self, order_id: str) -> OrderStatus | None:
        """Get the status of an order

        Args:
            order_id: ID of the order to check

        Returns:
            Order status or None if order not found
        """
        if not self.is_logged_in:
            self.logger.error("User not logged in")
            return None

        # TODO: Implement actual order status checking with Eastmoney API
        # This is a placeholder implementation
        self.logger.info(f"Checking status for order: {order_id}")
        return OrderStatus.FILLED

    def get_market_data(self, symbol: str) -> MarketData | dict[str, Any]:
        """Get market data for a symbol

        Args:
            symbol: Stock symbol

        Returns:
            Dictionary containing market data
        """
        if not self.is_logged_in:
            self.logger.warning("User not logged in")
            return {}

        # TODO: Implement actual market data retrieval with Eastmoney API
        # This is a placeholder implementation
        self.logger.info(f"Retrieving market data for: {symbol}")
        return {
            "symbol": symbol,
            "last_price": 100.0,
            "bid_price": 99.9,
            "ask_price": 100.1,
            "volume": 1000000,
            "timestamp": "2025-08-23T10:00:00",
        }
